@@ -16,8 +16,6 @@ from time import sleep
 from threading import Thread
 from subprocess import CalledProcessError
 from textwrap import dedent
-from time import sleep
-from threading import Thread
 from unittest import TestCase, mock
 
 import pytest
@@ -381,9 +379,7 @@ def test_reset_in_length():
 
 
 class TestResetErrors(TestCase):
-
     def test_reset_redefine(self):
-
         @magics_class
         class KernelMagics(Magics):
             @line_magic
@@ -827,6 +823,16 @@ def test_timeit_return():
     assert res is not None
 
 
+def test_timeit_save():
+    """
+    test whether timeit -v save object
+    """
+
+    name = "some_variable_name"
+    _ip.run_line_magic("timeit", "-n10 -r10 -v %s 1" % name)
+    assert _ip.user_ns[name] is not None
+
+
 def test_timeit_quiet():
     """
     test quiet option of timeit magic
@@ -846,6 +852,16 @@ def test_timeit_invalid_return():
         _ip.run_line_magic("timeit", "return")
 
 
+def test_timeit_raise_on_interrupt():
+    ip = get_ipython()
+
+    with pytest.raises(KeyboardInterrupt):
+        thread = Thread(target=_interrupt_after_1s)
+        thread.start()
+        ip.run_cell_magic("timeit", "", "from time import sleep; sleep(2)")
+        thread.join()
+
+
 @dec.skipif(execution.profile is None)
 def test_prun_special_syntax():
     "Test %%prun with IPython special syntax"
@@ -914,8 +930,71 @@ def test_notebook_export_json():
         _ip.run_line_magic("notebook", "%s" % outfile)
 
 
-class TestEnv(TestCase):
+def test_notebook_export_json_with_output():
+    """Tests if notebook export correctly captures outputs, errors, display outputs, and stream outputs."""
+    pytest.importorskip("nbformat")
+    pytest.importorskip("nbclient")
+    import nbformat
+    from nbclient import NotebookClient
 
+    _ip = get_ipython()
+    _ip.history_manager.reset()
+    _ip.colors = "neutral"
+    _ip.execution_count = 1
+
+    try:
+        commands = [
+            "1/0",
+            "print('test')",
+            "display('test')",
+            "1+1",
+            "display('a'), display('b')",
+            "import sys\nprint('test', file=sys.stderr)",
+        ]
+
+        clean_nb = nbformat.v4.new_notebook(
+            cells=[nbformat.v4.new_code_cell(source=cmd) for cmd in commands]
+        )
+
+        with TemporaryDirectory() as td:
+            outfile = os.path.join(td, "nb.ipynb")
+            client = NotebookClient(
+                clean_nb,
+                timeout=600,
+                kernel_name="python3",
+                resources={"metadata": {"path": td}},
+                allow_errors=True,
+            )
+            client.execute()
+            nbformat.write(clean_nb, outfile)
+            expected_nb = nbformat.read(outfile, as_version=4)
+
+        for cmd in commands:
+            _ip.run_cell(cmd, store_history=True, silent=False)
+            print(f"\n{_ip.history_manager.outputs}\n")
+
+        with TemporaryDirectory() as td:
+            outfile = os.path.join(td, "nb.ipynb")
+            _ip.run_cell(f"%notebook {outfile}", store_history=True)
+            sleep(2)
+            actual_nb = nbformat.read(outfile, as_version=4)
+
+        assert len(actual_nb["cells"]) == len(commands)
+        assert len(expected_nb["cells"]) == len(commands)
+
+        for i, command in enumerate(commands):
+            actual = actual_nb["cells"][i]
+            expected = expected_nb["cells"][i]
+            assert expected["source"] == command
+            assert actual["source"] == expected["source"]
+            assert (
+                actual["outputs"] == expected["outputs"]
+            ), f"Outputs do not match for cell {i+1} with source {command!r}"
+    finally:
+        _ip.colors = "nocolor"
+
+
+class TestEnv(TestCase):
     def test_env(self):
         env = _ip.run_line_magic("env", "")
         self.assertTrue(isinstance(env, dict))
@@ -1609,6 +1688,16 @@ def test_timeit_arguments():
     _ip.run_line_magic("timeit", "-n1 -r1 a=('#')")
 
 
+def test_time_raise_on_interrupt():
+    ip = get_ipython()
+
+    with pytest.raises(KeyboardInterrupt):
+        thread = Thread(target=_interrupt_after_1s)
+        thread.start()
+        ip.run_cell_magic("time", "", "from time import sleep; sleep(2)")
+        thread.join()
+
+
 MINIMAL_LAZY_MAGIC = """
 from IPython.core.magic import (
     Magics,
